package dev.shadowsoffire.apotheosis.adventure.loot;

import net.minecraft.util.ExtraCodecs;
import net.minecraft.resources.ResourceLocation;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import java.util.function.BiFunction;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.kinds.App;
import java.util.function.Function;
import com.mojang.datafixers.util.Either;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import javax.annotation.Nullable;

public interface RarityClamp
{
    public static final Simple UNCLAMPED = new Simple((DynamicHolder<LootRarity>)RarityRegistry.INSTANCE.emptyHolder(), (DynamicHolder<LootRarity>)RarityRegistry.INSTANCE.emptyHolder());
    
    @Nullable
    LootRarity getMinRarity();
    
    @Nullable
    LootRarity getMaxRarity();
    
    default LootRarity clamp(final LootRarity rarity) {
        return rarity.clamp(this.getMinRarity(), this.getMaxRarity());
    }
    
    record Simple(DynamicHolder<LootRarity> min, DynamicHolder<LootRarity> max) implements RarityClamp {
        public static final Codec<Simple> STRING_CODEC;
        public static final Codec<Simple> MIN_MAX_CODEC;
        public static final Codec<Simple> CODEC;
        
        @Override
        public LootRarity getMinRarity() {
            return this.min.getOptional().orElse(null);
        }
        
        @Override
        public LootRarity getMaxRarity() {
            return this.max.getOptional().orElse(null);
        }
        
        static {
            STRING_CODEC = ExtraCodecs.m_184415_(() -> Codec.STRING.xmap(s -> {
                final DynamicHolder rarity = RarityRegistry.INSTANCE.holder(new ResourceLocation(s));
                return new Simple((DynamicHolder<LootRarity>)rarity, (DynamicHolder<LootRarity>)rarity);
            }, simple -> simple.min().getId().toString()));
            MIN_MAX_CODEC = ExtraCodecs.m_184415_(() -> RecordCodecBuilder.create(inst -> inst.group((App)RarityRegistry.INSTANCE.holderCodec().fieldOf("min").forGetter((Function)Simple::min), (App)RarityRegistry.INSTANCE.holderCodec().fieldOf("max").forGetter((Function)Simple::max)).apply((Applicative)inst, (BiFunction)Simple::new)));
            CODEC = Codec.either((Codec)Simple.STRING_CODEC, (Codec)Simple.MIN_MAX_CODEC).xmap(e -> e.map((Function)Function.identity(), (Function)Function.identity()), (Function)Either::right);
        }
    }
}
